unit NApp;

{$IMPORTEDDATA OFF}

interface

uses Windows, NForm;

type
  TApp = class
  private
    FName: WideString;
  public
    Accel: HACCEL;
    MainForm: TForm;
    ModelessDlg: HWND;
    constructor Create(const Name: WideString); virtual;
    function AlreadyExists: Boolean;
    procedure Init; virtual;
    procedure ActivateNext(Prev: Boolean);
    procedure Error(const Msg: WideString);
    procedure Fatal(Code: Integer = 1);
    procedure Quit(Code: Integer);
    procedure Run;
    property Name: WideString read FName;
  end;

var
  App: TApp;

implementation

uses Messages, NLib, NSys;

type
  TListWindowRec = record
    Count: Integer;
    Wnds: HWNDArray;
    ClassName: string;
  end;
  PListWindowRec = ^TListWindowRec;

function EnumWindowsProc(hWnd: HWND; lParam: LPARAM): BOOL; stdcall;
var
  buf: array [0..255] of Char;
begin
  with PListWindowRec(lParam)^ do begin
    if ClassName <> '' then begin
      FillChar(buf, SizeOf(buf), 0);
      GetClassName(hWnd, buf, SizeOf(buf));
      if buf <> ClassName then begin
        Result := True;
        Exit;
      end;
    end;
    Wnds[Count] := hWnd;
    Inc(Count);
    if Count = Length(Wnds) then
      SetLength(Wnds, Count + Count div 4);
  end;
  Result := True;
end;

function ListWindow(const ClassName: string): HWNDArray;
var
  r: TListWindowRec;
begin
  r.Count := 0;
  SetLength(r.Wnds, 1024);
  r.ClassName := ClassName;
  EnumWindows(@EnumWindowsProc, LPARAM(@r));
  SetLength(r.Wnds, r.Count);
  Result := r.Wnds;
end;

constructor TApp.Create(const Name: WideString);
begin
  FName := Name;
end;

procedure TApp.Init;
begin
end;

procedure TApp.Error(const Msg: WideString);
begin
  MessageBoxW(GetDesktopWindow, PWideChar(Msg), PWideChar(FName), MB_ICONEXCLAMATION);
end;

procedure TApp.Fatal(Code: Integer = 1);
begin
  Error(FormatException(Exception(ExceptObject)));
  Halt(Code);
end;

procedure TApp.Run;
var
  msg: TMsg;
  r: Integer;
begin
  while True do begin
    r := Integer(GetMessageW(msg, 0, 0, 0));
    if (r = 0) or (r = -1) then
      Break;
    if (ModelessDlg <> 0) and IsDialogMessageW(ModelessDlg, Msg) then
      Continue;
    if (Accel <> 0) and
        (TranslateAcceleratorW(MainForm.Handle, Accel, Msg) <> 0) then
      Continue;
    TranslateMessage(msg);
    DispatchMessageW(msg);
  end;
  ExitCode := msg.wParam;
end;

procedure TApp.Quit(Code: Integer);
begin
  PostQuitMessage(Code);
end;

function TApp.AlreadyExists: Boolean;
begin
  if CreateMutexW(nil, True, PWideChar(FName + 'Mutex')) = 0 then
    raise EOSError.Create;
  Result := GetLastError = ERROR_ALREADY_EXISTS;
end;

procedure TApp.ActivateNext(Prev: Boolean);
var
  i, cur: Integer;
  wnds: HWNDArray;
begin
  wnds := ListWindow(MainForm.ClassName);
  cur := 0;
  for i := 0 to High(wnds) do
    if wnds[i] = MainForm.Handle then begin
      cur := i;
      Break;
    end;
  if Prev then begin
    Dec(cur);
    if cur < 0 then
      cur := High(wnds);
  end else begin
    Inc(cur);
    if cur = Length(wnds) then
      cur := 0;
  end;
  if IsIconic(wnds[cur]) then
    PostMessageW(wnds[cur], WM_SYSCOMMAND, SC_RESTORE, 0);
  SetForegroundWindow(wnds[cur]);
  if not Prev then
    SetWindowPos(MainForm.Handle, wnds[High(wnds)], 0, 0, 0, 0,
        SWP_NOSIZE or SWP_NOMOVE or SWP_NOACTIVATE);
end;

end.
