unit Winsock2Sub;

{$IMPORTEDDATA OFF}

interface

uses Windows, WinAdd;

const
  WSADESCRIPTION_LEN = 256;
  WSASYS_STATUS_LEN = 128;
  MAX_PROTOCOL_CHAIN = 7;
  WSAPROTOCOL_LEN = 255;

type
  // inaddr.h
  IN_ADDR = record
    case Integer of
      0: (s_b1, s_b2, s_b3, s_b4: UCHAR); // packed
      1: (s_w1, s_w2: USHORT); // packed
      2: (S_addr: ULONG);
  end;

  // in6addr.h
  IN6_ADDR = record
    case Integer of
      0: (Byte: array [0..15] of UCHAR);
      1: (Word: array [0..7] of USHORT);
  end;

  // winsock2.h
  SOCKET_ = UINT_PTR;
  GROUP = Cardinal;

  // ws2tcpip.h
  socklen_t = Integer;

  // ws2def.h
  ADDRESS_FAMILY = USHORT;

  WSABUF = record
    len: ULONG;
    buf: PChar;
  end;
  LPWSABUF = ^WSABUF;

  WSAOVERLAPPED = OVERLAPPED;
  LPWSAOVERLAPPED = ^OVERLAPPED;

  LPWSAOVERLAPPED_COMPLETION_ROUTINE = procedure (dwError: DWORD;
    cbTransferred: DWORD; lpOverlapped: LPWSAOVERLAPPED; dwFlags: DWORD); stdcall;

  WSADATA = record
    wVersion: Word;
    wHighVersion: Word;
    szDescription: array [0..WSADESCRIPTION_LEN] of Char;
    szSystemStatus: array [0..WSASYS_STATUS_LEN] of Char;
    iMaxSockets: Word;
    iMaxUdpDg: Word;
    lpVendorInfo: PChar;
  end;
  LPWSADATA = ^WSADATA;

  SOCKADDR = record
    sa_family: ADDRESS_FAMILY;
    sa_data: array [0..13] of Char;
  end;
  PSOCKADDR = ^SOCKADDR;
  LPSOCKADDR = ^SOCKADDR;

  SOCKADDR_IN = record
    sin_family: ADDRESS_FAMILY;
    sin_port: USHORT;
    sin_addr: IN_ADDR;
    sin_zero: array [0..7] of Char;
  end;
  PSOCKADDR_IN= ^SOCKADDR_IN;

  PADDRINFOW = ^ADDRINFOW;
  ADDRINFOW = record
    ai_flags: Integer;
    ai_family: Integer;
    ai_socktype: Integer;
    ai_protocol: Integer;
    ai_addrlen: Cardinal; // size_t
    ai_canonname: PWSTR;
    ai_addr: PSOCKADDR;
    ai_next: PADDRINFOW;
  end;

  WSAPROTOCOLCHAIN = record
    ChainLen: Integer;
    ChainEntories: array [0..MAX_PROTOCOL_CHAIN-1] of DWORD;
  end;
  LPWSAPROTOCOLCHAIN = ^WSAPROTOCOLCHAIN;

  WSAPROTOCOL_INFOW = record
    dwServiceFlags1: DWORD;
    dwServiceFlags2: DWORD;
    dwServiceFlags3: DWORD;
    dwServiceFlags4: DWORD;
    dwProviderFlags: DWORD;
    ProviderId: TGUID;
    dwCatalogEntryId: DWORD;
    ProtocolChain: WSAPROTOCOLCHAIN;
    iVersion: Integer;
    iAddressFamily: Integer;
    iMaxSockAddr: Integer;
    iMinSockAddr: Integer;
    iSocketType: Integer;
    iProtocol: Integer;
    iProtocolMaxOffset: Integer;
    iNetworkByteOrder: Integer;
    iSecurityScheme: Integer;
    dwMessageSize: DWORD;
    dwProviderReserved: DWORD;
    szProtocol: array [0..WSAPROTOCOL_LEN] of WideChar;
  end;
  LPWSAPROTOCOL_INFOW = ^WSAPROTOCOL_INFOW;

  // ws2ipdef.h
  SOCKADDR_IN6 = record
    sin6_family: ADDRESS_FAMILY;
    sin6_port: USHORT;
    sin6_flowinfo: ULONG;
    sin6_addr: IN6_ADDR;
    sin6_scope_id: ULONG;
  end;
  PSOCKADDR_IN6 = ^SOCKADDR;
  LPSOCKADDR_IN6 = ^SOCKADDR;

const
  // ws2def.h
  AF_INET = 2;
  AF_INET6 = 23;
  SOCK_DGRAM = 2;
  IPPROTO_UDP = 17;
  NI_MAXHOST = 1025;

  // winsock2.h
  INVALID_SOCKET = SOCKET_(not 0);
  SOCKET_ERROR = -1;
  WSA_FLAG_OVERLAPPED = $01;
  WSA_IO_PENDING = ERROR_IO_PENDING;

  // ws2ipdef.h
  INET_ADDRESTRLEN = 22;
  INET6_ADDRSTRLEN = 65;

  ws2_32 = 'ws2_32.dll';

function WSAStartup(wVersionRequired: Word; lpWSAData: LPWSADATA): Integer;
    stdcall; external ws2_32 name 'WSAStartup';
function WSACleanup: Integer;
    stdcall; external ws2_32 name 'WSACleanup';
function WSAGetLastError: Integer;
    stdcall; external ws2_32 name 'WSAGetLastError';
function GetAddrInfoW(pNodeName: PCWSTR; pServiceName: PCWSTR;
    const pHints: PADDRINFOW; var pResult: PADDRINFOW): Integer;
    stdcall; external ws2_32 name 'GetAddrInfoW';
procedure FreeAddrInfoW(pAddrInfo: PADDRINFOW);
    stdcall; external ws2_32 name 'FreeAddrInfoW';
function GetNameInfoW(const pSockaddr: PSOCKADDR; SockaddrLength: socklen_t;
    pNodeBuffer: PWCHAR; NodeBufferSize: DWORD; pServiceBuffer: PWCHAR;
    ServiceBufferSize: DWORD; Flags: Integer): Integer;
    stdcall; external ws2_32 name 'GetNameInfoW';
function GetHostNameW(name: PWSTR; namelen: Integer): Integer;
    stdcall; external ws2_32 name 'GetHostNameW';

function WSASocketW(af: Integer; type_: Integer; protocol: Integer;
    lpProtocolInfo: LPWSAPROTOCOL_INFOW; g: GROUP; dwFlags: DWORD): SOCKET_;
    stdcall; external ws2_32 name 'WSASocketW';
function WSARecvFrom(s: SOCKET_; lpBuffers: LPWSABUF; dwBufferCount: DWORD;
    lpNumberOfBytesRecvd: LPDWORD; lpFlags: LPDWORD; lpFrom: PSOCKADDR;
    lpFromlen: LPINT; lpOverlapped: LPWSAOVERLAPPED;
    lpCompleteRoutine: LPWSAOVERLAPPED_COMPLETION_ROUTINE): Integer;
    stdcall; external ws2_32 name 'WSARecvFrom';
function WSAGetOverlappedResult(s: SOCKET_; lpOverlapped: LPWSAOVERLAPPED;
    lpcbTransger: LPDWORD; fWait: BOOL; lpdwFlags: LPDWORD): BOOL;
    stdcall; external ws2_32 name 'WSAGetOverlappedResult';

// IPv6 support: XP SP1 and later
function WSAAddressToStringW(lpsaAddress: LPSOCKADDR; dwAddressLength: DWORD;
    lpProtocolInfo: LPWSAPROTOCOL_INFOW; lpszAddressString: LPWSTR;
    lpdwAddressStringLength: LPDWORD): Integer;
    stdcall; external ws2_32 name 'WSAAddressToStringW';

function socket(af: Integer; type_: Integer; protocol: Integer): SOCKET_;
    stdcall; external ws2_32 name 'socket';
function closesocket(s: SOCKET_): Integer;
    stdcall; external ws2_32 name 'closesocket';
function bind(s: SOCKET_; name: PSOCKADDR; namelen: Integer): Integer;
    stdcall; external ws2_32 name 'bind';
function sendto(s: SOCKET_; const buf: PChar; len: Integer; flags: Integer;
    to_: PSOCKADDR; tolen: Integer): Integer;
    stdcall; external ws2_32 name 'sendto';
function recvfrom(s: SOCKET_; buf: PChar; len: Integer; flags: Integer;
    from: PSOCKADDR; formlen: PInteger): Integer;
    stdcall; external ws2_32 name 'recvfrom';

implementation
end.
